"""
Token使用统计系统的核心数据模型

定义强类型数据对象，用于在系统中传递token使用相关信息
"""

from dataclasses import dataclass, field
from typing import Dict, List, Optional, Any, Protocol, Union, cast, TypeGuard
from datetime import datetime

from agentlang.logger import get_logger

logger = get_logger(__name__)


class PromptTokensDetails(Protocol):
    """LLM响应中的prompt_tokens_details对象协议

    注意：必须使用属性访问，如 details.cache_write_input_tokens
    不得使用字典方式访问，如 details.get('cache_write_input_tokens')
    """
    cache_write_input_tokens: int  # 写入缓存的token数
    cached_tokens: int  # 命中缓存的token数


class LlmUsageInfo(Protocol):
    """LLM响应中的usage信息协议

    注意：必须使用属性访问，如 usage.prompt_tokens
    不得使用字典方式访问，如 usage.get('prompt_tokens')
    """
    prompt_tokens: int  # 输入token数
    completion_tokens: int  # 输出token数
    prompt_tokens_details: Optional[PromptTokensDetails]  # 缓存相关token详情


def is_prompt_tokens_details(obj: Any) -> TypeGuard[PromptTokensDetails]:
    """检查对象是否符合PromptTokensDetails协议

    Args:
        obj: 要检查的对象

    Returns:
        bool: 是否符合协议
    """
    return (
        hasattr(obj, "cache_write_input_tokens") and
        hasattr(obj, "cached_tokens")
    )


def is_llm_usage_info(obj: Any) -> TypeGuard[LlmUsageInfo]:
    """检查对象是否符合LlmUsageInfo协议

    Args:
        obj: 要检查的对象

    Returns:
        bool: 是否符合协议
    """
    return (
        hasattr(obj, "prompt_tokens") and
        hasattr(obj, "completion_tokens")
    )


@dataclass
class CacheUsage:
    """缓存使用量数据"""
    write_tokens: int = 0  # 写入缓存的token数
    hit_tokens: int = 0    # 命中缓存的token数


@dataclass
class TokenUsage:
    """基本token使用量数据"""
    input_tokens: int = 0
    output_tokens: int = 0
    cache: CacheUsage = field(default_factory=CacheUsage)

    @property
    def total_tokens(self) -> int:
        """计算总token数量"""
        return self.input_tokens + self.output_tokens + self.cache.write_tokens + self.cache.hit_tokens

    @classmethod
    def from_response(cls, response_usage: Any, model_id: str) -> 'TokenUsage':
        """从LLM响应创建对象

        Args:
            response_usage: LLM响应中的usage对象
            model_id: 模型ID

        Returns:
            TokenUsage: 标准化的token使用量对象
        """
        if not response_usage:
            return cls()

        try:
            # 根据对象类型采用不同的访问方式
            if isinstance(response_usage, dict):
                # 字典访问方式
                input_tokens = response_usage.get("prompt_tokens", 0)
                output_tokens = response_usage.get("completion_tokens", 0)

                # 初始化缓存数据
                cache_write_tokens = 0
                cached_tokens = 0

                # 提取缓存数据
                details = response_usage.get("prompt_tokens_details")
                if details and isinstance(details, dict):
                    cache_write_tokens = details.get("cache_write_input_tokens", 0)
                    cached_tokens = details.get("cached_tokens", 0)
            else:
                # 属性访问方式
                input_tokens = getattr(response_usage, "prompt_tokens", 0)
                output_tokens = getattr(response_usage, "completion_tokens", 0)

                # 初始化缓存数据
                cache_write_tokens = 0
                cached_tokens = 0

                # 提取缓存数据
                if hasattr(response_usage, "prompt_tokens_details"):
                    details = response_usage.prompt_tokens_details
                    if details:
                        if isinstance(details, dict):
                            # 字典类型的details
                            cache_write_tokens = details.get("cache_write_input_tokens", 0)
                            cached_tokens = details.get("cached_tokens", 0)
                        else:
                            # 对象类型的details，使用属性访问
                            cache_write_tokens = getattr(details, "cache_write_input_tokens", 0)
                            cached_tokens = getattr(details, "cached_tokens", 0)

            # 创建缓存使用对象
            cache = CacheUsage(
                write_tokens=cache_write_tokens,
                hit_tokens=cached_tokens
            )

            return cls(input_tokens, output_tokens, cache)
        except Exception as e:
            logger.error(f"从响应中提取token使用情况时出错: {e}")
            return cls()


@dataclass
class ModelUsage:
    """单个模型的使用情况"""
    model_name: str
    usage: TokenUsage
    cost: float = 0.0
    currency: str = "CNY"


@dataclass
class CostReport:
    """成本报告对象"""
    models: List[ModelUsage] = field(default_factory=list)
    timestamp: str = field(default_factory=lambda: datetime.now().strftime("%Y-%m-%d %H:%M:%S"))
    currency_code: str = "CNY"

    @property
    def total_input_tokens(self) -> int:
        """计算总输入token数"""
        return sum(model.usage.input_tokens for model in self.models)

    @property
    def total_output_tokens(self) -> int:
        """计算总输出token数"""
        return sum(model.usage.output_tokens for model in self.models)

    @property
    def total_cache_write_tokens(self) -> int:
        """计算总缓存写入token数"""
        return sum(model.usage.cache.write_tokens for model in self.models)

    @property
    def total_cache_hit_tokens(self) -> int:
        """计算总缓存命中token数"""
        return sum(model.usage.cache.hit_tokens for model in self.models)

    @property
    def total_tokens(self) -> int:
        """计算总token数"""
        return sum(model.usage.total_tokens for model in self.models)

    @property
    def total_cost(self) -> float:
        """计算总成本"""
        return sum(model.cost for model in self.models)


def get_currency_symbol(currency_code: str) -> str:
    """获取货币符号

    Args:
        currency_code: 货币代码

    Returns:
        str: 货币符号
    """
    currency_symbols = {
        "CNY": "¥",
        "USD": "$",
        "EUR": "€",
        "GBP": "£",
        "JPY": "¥"
    }
    return currency_symbols.get(currency_code, currency_code)
