"""
Token使用跟踪模块

提供LLM请求Token使用情况的跟踪功能
"""

import threading
import copy
from typing import Any, Dict, Optional, Protocol, cast
from datetime import datetime
import os

from agentlang.llms.token_usage.models import (
    TokenUsage, CacheUsage, LlmUsageInfo, PromptTokensDetails,
    is_llm_usage_info, is_prompt_tokens_details
)
from agentlang.logger import get_logger

# 延迟导入TokenUsageReport，避免循环引用
from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from agentlang.llms.token_usage.report import TokenUsageReport


logger = get_logger(__name__)


class LlmUsageResponse(Protocol):
    """LLM使用量响应的协议类型"""
    recorded: bool
    model_id: str
    model_name: str
    input_tokens: int
    output_tokens: int
    total_tokens: int
    cache_write_tokens: Optional[int]
    cache_hit_tokens: Optional[int]


class TokenUsageTracker:
    """Token用量跟踪器

    跟踪LLM请求的token用量，支持多线程安全的累计统计
    负责从LLM响应中提取token使用信息
    使用标准化的TokenUsage对象统一数据格式
    """

    def __init__(self) -> None:
        """初始化Token用量跟踪器"""
        # 使用字典记录每个模型的TokenUsage对象
        self._usage: Dict[str, TokenUsage] = {}
        # 使用锁确保线程安全
        self._lock = threading.Lock()
        # 报告管理器
        self._report_manager: Optional['TokenUsageReport'] = None

    def add_usage(self, model_id: str, token_usage: TokenUsage) -> None:
        """添加token使用记录

        Args:
            model_id: 模型ID
            token_usage: TokenUsage对象
        """
        with self._lock:
            if model_id not in self._usage:
                self._usage[model_id] = TokenUsage()

            # 累加使用量
            self._usage[model_id].input_tokens += token_usage.input_tokens
            self._usage[model_id].output_tokens += token_usage.output_tokens
            self._usage[model_id].cache.write_tokens += token_usage.cache.write_tokens
            self._usage[model_id].cache.hit_tokens += token_usage.cache.hit_tokens

    def record_llm_usage(self, response_usage: Any, model_id: str,
                         user_id: Optional[str] = None,
                         model_name: Optional[str] = None) -> LlmUsageResponse:
        """记录LLM使用情况，并生成报告

        Args:
            response_usage: LLM响应对象的usage属性
            model_id: 模型ID
            user_id: 用户ID，可选
            model_name: 模型名称，可选

        Returns:
            LlmUsageResponse: 记录结果
        """
        # 从响应中提取token使用情况
        token_usage = TokenUsage.from_response(response_usage, model_id)

        # 添加使用记录到跟踪器
        self.add_usage(model_id, token_usage)

        # 如果有报告管理器，则生成报告
        if self._report_manager:
            self._report_manager.update_and_save_usage(model_id, token_usage)

        # 返回处理结果，实现LlmUsageResponse协议
        class UsageResult:
            def __init__(self, usage: TokenUsage, mid: str, name: str):
                self.recorded = True
                self.model_id = mid
                self.model_name = name or mid
                self.input_tokens = usage.input_tokens
                self.output_tokens = usage.output_tokens
                self.total_tokens = usage.total_tokens
                self.cache_write_tokens = usage.cache.write_tokens if usage.cache.write_tokens > 0 else None
                self.cache_hit_tokens = usage.cache.hit_tokens if usage.cache.hit_tokens > 0 else None

        return UsageResult(token_usage, model_id, model_name)

    def extract_chat_history_usage_data(self, chat_response: Any) -> TokenUsage:
        """从LLM响应对象中提取处理好的token使用数据，用于chat_history

        为agent.py提供的便捷方法，从LLM响应中提取token使用数据但不记录到使用统计中

        Args:
            chat_response: 完整的LLM响应对象

        Returns:
            TokenUsage: 标准化的token使用量对象，如果无法提取则返回空对象
        """
        # 检查响应对象是否有效
        if not chat_response:
            return TokenUsage()

        try:
            # 提取usage属性，注意使用属性访问
            usage = getattr(chat_response, "usage", None)
            if not usage:
                return TokenUsage()

            # 使用TokenUsage提取标准化的数据
            return TokenUsage.from_response(usage, "")
        except Exception as e:
            logger.error(f"提取chat_history token使用数据失败: {e}", exc_info=True)
            return TokenUsage()

    def set_report_manager(self, report_manager: 'TokenUsageReport') -> None:
        """设置报告管理器

        Args:
            report_manager: TokenUsageReport 实例
        """
        self._report_manager = report_manager

    def get_usage_data(self) -> Dict[str, TokenUsage]:
        """获取所有使用数据

        Returns:
            Dict[str, TokenUsage]: 所有模型的使用数据
        """
        with self._lock:
            # 返回深拷贝避免外部修改
            return copy.deepcopy(self._usage)

    def reset(self) -> None:
        """重置所有使用统计"""
        with self._lock:
            self._usage.clear()
            logger.info("TokenUsageTracker已重置")

    def get_formatted_report(self) -> str:
        """获取格式化的报告（一步到位）

        Returns:
            str: 格式化后的token使用报告
        """
        if not self._report_manager:
            # 如果没有报告管理器，使用简单格式输出当前累计数据
            lines = ["Token使用统计（临时报告）："]

            with self._lock:
                if not self._usage:
                    return "无Token使用记录"

                for model_name, usage in self._usage.items():
                    lines.append(f"\n模型: {model_name}")
                    lines.append(f"  输入tokens: {usage.input_tokens:,}")
                    lines.append(f"  输出tokens: {usage.output_tokens:,}")

                    if usage.cache.write_tokens > 0:
                        lines.append(f"  缓存写入tokens: {usage.cache.write_tokens:,}")
                    if usage.cache.hit_tokens > 0:
                        lines.append(f"  缓存命中tokens: {usage.cache.hit_tokens:,}")

                    lines.append(f"  总tokens: {usage.total_tokens:,}")

            return "\n".join(lines)
        else:
            # 使用报告管理器生成完整报告
            cost_report = self._report_manager.get_cost_report()
            return self._report_manager.format_report(cost_report)
